# -*- coding: utf-8 -*-

from tuikit.widget import Widget
from tuikit.common import Borders

import logging


class Container(Widget):

    '''Container widget. Base for any widget which can contain other widgets.'''

    def __init__(self, width = 10, height = 10):
        '''Create container of requested size.'''
        Widget.__init__(self, width, height)

        #: List of child widgets.
        self.children = []

        self.focuschild = None
        self.mousechild = None

        self.allow_focus = True

        #: Width of borders (left, top, right, bottom).
        #: Child widgets are placed within borders.
        self.borders = Borders()

        self._layout = None

        self.widthrequest = (None, None)
        self.heightrequest = (None, None)

        self.colorprefix = None

        self.trapfocus = False  # if True, tab cycles inside container


    def add(self, widget, **kw):
        '''Add widget into this container.'''
        self.children.append(widget)
        widget.parent = self
        widget.settop(self.top)
        widget.hints.update(kw)
        if self.focuschild is None and widget.canfocus():
            self.focuschild = widget


    def layout(self, layout):
        '''Set layout manager for placing child widgets.'''
        self._layout = layout
        layout.container = self
        self.connect('resize', layout.resize)


    def settop(self, top):
        '''Set top widget.'''
        self.top = top
        for child in self.children:
            child.settop(top)


    def focusnext(self):
        idx_current = self.children.index(self.focuschild)
        idx_new = idx_current + 1
        cycled = False
        while idx_current != idx_new:
            if idx_new >= len(self.children):
                idx_new = 0
                cycled = True
            if self.children[idx_new].canfocus():
                self.children[idx_new].setfocus()
                return self.trapfocus or not cycled
            idx_new += 1

    def on_keypress(self, keyname, char):
        super().on_keypress(keyname, char)
        if self.focuschild is not None:
            handled = self.focuschild.emit('keypress', keyname, char)
            if handled:
                return True
        if keyname == 'tab':
            return self.focusnext()


    def on_resize(self):
        super().on_resize()
        for child in self.children:
            child.emit('resize')


    def draw(self, driver, x, y):
        if self.hidden:
            return True

        driver.clipstack.push(x, y, self.width, self.height)
        if self.colorprefix:
            driver.pushcolorprefix(self.colorprefix)

        super().draw(driver, x, y)

        for child in [x for x in self.children if not x.allowlayout]:
            child.draw(driver, x + child.x, y + child.y)

        l, t, r, b = self.borders
        driver.clipstack.push(x+l, y+t, self.width-l-r, self.height-t-b)

        for child in [x for x in self.children if x.allowlayout]:
            child.draw(driver, x + child.x, y + child.y)

        driver.clipstack.pop()

        if self.colorprefix:
            driver.popcolorprefix()
        driver.clipstack.pop()


    def on_mousedown(self, ev):
        super().on_mousedown(ev)
        handled = False
        for child in reversed(self.children):
            if child.enclose(ev.wx, ev.wy):
                childev = ev.childevent(child)
                child.emit('mousedown', childev)
                self.mousechild = child
                handled = True
                break
        return handled

    def on_mouseup(self, ev):
        super().on_mouseup(ev)
        if self.mousechild:
            childev = ev.childevent(self.mousechild)
            self.mousechild.emit('mouseup', childev)
            self.mousechild = None
            return True

    def on_mousemove(self, ev):
        super().on_mousemove(ev)
        if self.mousechild:
            childev = ev.childevent(self.mousechild)
            self.mousechild.emit('mousemove', childev)
            return True

    def on_mousewheel(self, ev):
        super().on_mousewheel(ev)
        handled = False
        for child in reversed(self.children):
            if child.enclose(ev.wx, ev.wy):
                childev = ev.childevent(child)
                child.emit('mousewheel', childev)
                handled = True
                break
        return handled

